#include <stdint.h>
#include <cpu.h>
#include <memory.h>
#include <vga.h>
#include <debug.h>
#include <multiboot.h>
#include <interrupts.h>

struct kernel_boot_data_st kernel_boot_data;

int divide(int a, int b)
{
    return a/b;
}

registers *divbyzero(registers *r) {
    PANIC("DIVIDE BY ZERO!");
    return r;
}

void kmain(uint64_t multiboot_magic, void *multiboot_data)
{
    vga_init();

    debug("Started kernel\n");

    cpu_init();

    multiboot_init(multiboot_magic, P2V(multiboot_data));

    debug_info("Kernel loaded with command line: \"%s\" by <%s>\n", kernel_boot_data.commandline, kernel_boot_data.bootloader);

    debug_info("Memory areas:\n");
    size_t index = 0;
    uintptr_t start, end;
    uint32_t type;
    while(!multiboot_get_memory_area(index++, &start, &end, &type))
    {
        debug_info("%d %x-%x\n", type, start, end);
    }
    bind_interrupt(0, divbyzero);

    debug("Hello, world!");

    // Force a divide by zero error
    divide(5, 0);

    debug_info("Boot complete\n");

    PANIC("End of kernel function!");
    
    debug_info("Broke out of panic");
    for(;;);
}